package com.gct.tools.etlcamelhuge.routeconfig;

import com.gct.common.util.SGTUtil;
import com.gct.tools.etlcamelhuge.MQ.MessageProducer;
import com.gct.tools.etlcamelhuge.entity.DiagnoseMsg;
import org.apache.camel.Message;
import org.apache.camel.builder.RouteBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.Reader;
import java.lang.management.MemoryMXBean;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Clob;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * class name: CamelJDBCCofRealTimeConfiguration.java
 * 实时导数据类
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/4/14 下午3:16
 */
@Configuration
public class CamelJDBCCofRealTimeConfiguration {

    private static long sendMsgRunTime = 0;

    public Double min(String[] strings) {
        double[] doubles = new double[strings.length];
        for (int i = 0; i < strings.length; i++) {
            doubles[i] = Double.parseDouble(strings[i]);
        }
        OptionalDouble min = Arrays.stream(doubles).min();
        return min.isPresent() ? min.getAsDouble() : null;
    }


    //获取最大载荷
    public Double max(String[] strings) {
        double[] doubles = new double[strings.length];
        for (int i = 0; i < strings.length; i++) {
            doubles[i] = Double.parseDouble(strings[i]);
        }
        OptionalDouble max = Arrays.stream(doubles).max();
        return max.isPresent() ? max.getAsDouble() : null;
    }


    public String getDate() {
        return LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }


    /***
     * 异步发送 RunTime 消息到MQ中
     * */
    // @Async
    public void sendDataToRocketMQ(String wellName, String wellId, String prodDate, Double stroke_length, Double stroke_frequency, String sgt) {
        String orgId = "0";
        DiagnoseMsg diagnoseMsg = new DiagnoseMsg(wellId, wellName, orgId, prodDate, sgt, LocalDateTime.now().toString(), stroke_length, stroke_frequency);
//        System.out.println(diagnoseMsg);
        sendMsgRunTime++;
        producer.send(diagnoseMsg);
    }

    @Resource(name = "diagnoseMessageProducer")
    private MessageProducer producer;

    private String clobToString(Clob clob){
        StringBuilder sb = new StringBuilder();
        if (clob != null) {
            try (Reader reader = clob.getCharacterStream()) {
                char[] buffer = new char[1024];
                int bytesRead;
                while ((bytesRead = reader.read(buffer)) != -1) {
                    sb.append(buffer, 0, bytesRead);
                }
            } catch (SQLException e) {
                throw new RuntimeException(e);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        return sb.toString();

    }

    @Bean
    public RouteBuilder routeBuilderWithRealTime() {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                from("timer:mytimer-insert-runtime?period=1800000")
                        .routeId("insert-runtime")
                        .setBody(simple("select well_id ,sgt_last_time  from centralbase.sys_access_well_control  where  access_status='1'  "))
                        .to("jdbc:centralbase")
                        .split(body()).process(exchange -> {
                            HashMap body = exchange.getIn().getBody(HashMap.class);
                            exchange.getIn().setHeader("well_id", body.get("well_id"));
//                            System.out.println(body.get("well_id"));
                            if (Objects.isNull(body.get("sgt_last_time")))body.put("sgt_last_time"
                                    ,LocalDateTime.now().minusHours(4).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                            exchange.getIn().setHeader("sgt_last_time", body.get("sgt_last_time").toString().substring(0,19));
                        })
                        .setBody(simple("select WELL_ID,WELL_NAME,DYNA_CREATE_TIME,CHECK_DATE,DISPLACEMENT,DISP_LOAD,STROKE,FREQUENCY,SUSP_MAX_LOAD,SUSP_MIN_LOAD" +
                                " from DEFAULT_GONGTU where CHECK_DATE >= to_date('${header.sgt_last_time}','yyyy-mm-dd hh24:mi:ss') " +
                                "and well_id='${header.well_id}' and  rownum<50 "))
                        .to("jdbc:oracle")
                        .process(exchange -> {
                            sendMsgRunTime = 0;
                        })
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            String prod_date = aRow.get("DYNA_CREATE_TIME").toString().split("\\+")[0];
                            aRow.put("DYNA_CREATE_TIME", prod_date);
                            aRow.put("sgt", "");
                            //swap table error design column
                            Object tempObj = aRow.get("STROKE");
                            aRow.put("STROKE",aRow.get("FREQUENCY"));
                            aRow.put("FREQUENCY",tempObj);
                            if (!StringUtils.isEmpty(aRow.get("DISPLACEMENT")) && !StringUtils.isEmpty(aRow.get("DISP_LOAD"))) {
                                String[] displacements =  clobToString((Clob) aRow.get("DISPLACEMENT")).split(",");//10 四舍五入
                                for (int i = 0; i < displacements.length; i++) {
                                    displacements[i] = Double.toString(Double.parseDouble(displacements[i])/10);
                                }
//                                System.out.println("disp="+Arrays.deepToString(displacements));
                                String[] disp_loads = clobToString((Clob) aRow.get("DISP_LOAD")).split(",");
                                Double SUSP_MAX_LOAD = max(disp_loads);
                                Double SUSP_MIN_LOAD = min(disp_loads);
                                String sgt = "";
                                for (int i = 0; i < displacements.length; i++) {
                                    sgt = sgt + displacements[i] + "," + disp_loads[i] + ",";
                                }
                                String[] s = sgt.split(",");
                                String w = "";
                                for (int i = 0; i < s.length; i++) {
                                    w += new BigDecimal(Math.round(Double.parseDouble(s[i]) * 100)).stripTrailingZeros().toPlainString() + ",";
                                }
                                Double[][] doubles = SGTUtil.encodeToDoubleArray(w);
                                aRow.put("sgt", SGTUtil.encodeToString(doubles));
                                aRow.put("SUSP_MAX_LOAD", SUSP_MAX_LOAD);
                                aRow.put("SUSP_MIN_LOAD", SUSP_MIN_LOAD);
                            }
                            //对于位移没有数据,所有数据都在载荷中的特殊数据做特别处理
                            if (StringUtils.isEmpty(aRow.get("DISPLACEMENT")) && !StringUtils.isEmpty(aRow.get("DISP_LOAD"))) {
                                String DISP_LOAD = aRow.get("DISP_LOAD").toString().replaceAll(";", ",");
                                Double[][] doubles = SGTUtil.encodeToDoubleArray(DISP_LOAD);
                                aRow.put("sgt", SGTUtil.encodeToString(doubles));
                                String[] split = DISP_LOAD.split(",");
                                List<String> list = new ArrayList<>();
                                for (int i = 0; i < split.length; i++) {
                                    if (i % 2 != 0) {
                                        BigDecimal bigDecimal = new BigDecimal(split[i]).divide(new BigDecimal("100.0"));
                                        list.add(bigDecimal.toString());
                                    }
                                }
                                String[] loads = list.toArray(new String[0]);
                                Double SUSP_MAX_LOAD = null;
                                Double SUSP_MIN_LOAD = null;
                                if (loads.length > 0) {
                                    SUSP_MAX_LOAD = max(loads);
                                    SUSP_MIN_LOAD = min(loads);
                                }
                                aRow.put("SUSP_MAX_LOAD", SUSP_MAX_LOAD);
                                aRow.put("SUSP_MIN_LOAD", SUSP_MIN_LOAD);
                            }
                            aRow.putIfAbsent("STROKE", "0.0");
                            aRow.putIfAbsent("FREQUENCY", "0.0");
                            aRow.putIfAbsent("SUSP_MAX_LOAD", "0.0");
                            aRow.putIfAbsent("SUSP_MIN_LOAD", "0.0");
                            if (!StringUtils.isEmpty(aRow.get("FREQUENCY"))) {
                                BigDecimal bd = new BigDecimal(aRow.get("FREQUENCY").toString());
                                double FREQUENCY = bd.setScale(1, RoundingMode.HALF_UP).doubleValue();
                                aRow.put("FREQUENCY", FREQUENCY);
                            }
                            if (!StringUtils.isEmpty(aRow.get("STROKE"))) {
                                double stroke1 = Double.parseDouble(aRow.get("STROKE").toString());
                                BigDecimal bd = new BigDecimal(stroke1);
                                double STROKE = bd.setScale(1, RoundingMode.HALF_UP).doubleValue();
                                aRow.put("STROKE", STROKE);
                            }
                            String wellName = aRow.get("WELL_NAME").toString();
                            String wellId = aRow.get("WELL_ID").toString();
                            String prodDate = aRow.get("DYNA_CREATE_TIME").toString().substring(0, 19);
                            Double strokeLength = Double.valueOf(aRow.get("STROKE").toString());
                            Double strokeFrequency = Double.valueOf(aRow.get("FREQUENCY").toString());
                            String sgt = aRow.get("sgt").toString();
//                            System.out.println(sgt);
                            in.setHeader("sgt_last_time", prodDate);
                            in.setHeader("well_id", wellId);
                            sendDataToRocketMQ(wellName, wellId, prodDate, strokeLength, strokeFrequency, sgt);
                        })
                        .setBody(simple("insert into centralbase.cb_temp_well_mech_runtime(well_id,prod_date,SUSP_MAX_LOAD,SUSP_MIN_LOAD,sgt,stroke_length,stroke_frequency) " +
                                "select '${body[WELL_ID]}','${body[DYNA_CREATE_TIME]}','${body[SUSP_MAX_LOAD]}','${body[SUSP_MIN_LOAD]}','${body[sgt]}','${body[STROKE]}','${body[FREQUENCY]}' " +
                                "where NOT EXISTS (SELECT * FROM centralbase.cb_temp_well_mech_runtime WHERE well_id = '${body[WELL_ID]}' and  prod_date = '${body[DYNA_CREATE_TIME]}' )"))
                        .to("jdbc:centralbase")
                        .setBody(simple("update centralbase.sys_access_well_control set sgt_last_time = '${header.sgt_last_time}' where well_id ='${header.well_id}' "))
                        .to("jdbc:centralbase")
                        .end();

                from("timer:mytimer-update-avg-mech_daily?period=10800000")
                        .routeId("update-avg-mech_daily")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        })
                        .setBody(simple("select well_id,avg(stroke_length) stroke_length  ,avg(stroke_frequency) stroke_frequency from centralbase.cb_temp_well_mech_runtime where prod_date::date='${header.date}' group by well_id"))
                        .to("jdbc:centralbase")
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            if (aRow.get("stroke_length") != null && aRow.get("stroke_frequency") != null) {
                                double stroke_length = Double.parseDouble(aRow.get("stroke_length").toString());
                                double stroke_frequency = Double.parseDouble(aRow.get("stroke_frequency").toString());
                                BigDecimal bd = new BigDecimal(stroke_length);
                                double stroke_lengt1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                BigDecimal bd1 = new BigDecimal(stroke_frequency);
                                double stroke_frequency1 = bd1.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                aRow.put("strokeLength", stroke_lengt1);
                                aRow.put("strokeFrequency", stroke_frequency1);
                            }
                        })
                        .setBody(simple("update centralbase.cb_temp_well_mech_daily set stroke_length='${body[strokeLength]}' ,stroke_frequency ='${body[strokeFrequency]}' where well_id = '${body[well_id]}' and prod_date::date='${header.date}' "))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-avg-mech_daily ->  centralbase.cb_temp_well_mech_daily update data failed")
                        .end();

               /* from("timer:mytimer-insert-mechDaily?period=3600000")
                        .routeId("insert-mech_daily")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        })
                        .setBody(simple("select well_id from centralbase.sys_access_well_control  where  access_status='1'  "))
                        .to("jdbc:centralbase")
                        .split(body()).process(exchange -> {
                            HashMap body = exchange.getIn().getBody(HashMap.class);
                            exchange.getIn().setHeader("well_id", body.get("well_id"));
                        })
                        .setBody(simple("select distinct jh,rq,dym,jy,ly,bj,bs,bx,zs,cc,cs,blx,dl from DBA01 where rq  = to_date('${header.date}','yyyy-MM-dd') and jh='${header.well_id}' and qyrq is not null  "))
                        .to("jdbc:oracle")
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            if (aRow.get("JY") == null) aRow.put("JY", "0.0");
                            if (aRow.get("LY") == null) aRow.put("LY", "0.0");
                            if (aRow.get("BJ") == null) aRow.put("BJ", "0.0");
                            if (aRow.get("BS") == null) aRow.put("BS", "0.0");
                            if (aRow.get("BX") == null) aRow.put("BX", "0.0");
                            if (aRow.get("ZS") == null) aRow.put("ZS", "0.0");
                            if (aRow.get("CC") == null) aRow.put("CC", "0.0");
                            if (aRow.get("CS") == null) aRow.put("CS", "0.0");
                            if (aRow.get("BLX") == null) aRow.put("BLX", "");
                            if (aRow.get("DL") == null) aRow.put("DL", "0.0");
                        })
                        .setBody(simple("insert into centralbase.cb_temp_well_mech_daily(well_id,prod_date,static_pressure,flow_pres,pump_diameter,pump_depth,pump_efficiency,rotate_frequency,stroke_length,stroke_frequency,pump_type,elec_frequency) " +
                                "select '${body[JH]}','${body[RQ]}','${body[JY]}','${body[LY]}','${body[BJ]}','${body[BS]}','${body[BX]}','${body[ZS]}','${body[CC]}','${body[CS]}','${body[BLX]}','${body[DL]}' " +
                                "where NOT EXISTS ( SELECT * FROM centralbase.cb_temp_well_mech_daily WHERE well_id = '${body[JH]}' and  prod_date = '${body[RQ]}' )"))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:insert-mech_daily ->  centralbase.cb_temp_well_mech_daily insert data failed")
                        .end();*/
            }

        };
    }
}
